package footyquiz.server;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import footyquiz.server.integration.HighscoreDAO;
import footyquiz.server.integration.QuestionDAO;

/**
 * Business logics for Footyquiz server.
 * 
 * @author Mikael Kopteff & Hannu Leinonen
 */
public class Command {

	private final String TEXT = "text";
	private final String WRONG1 = "wrong_1";
	private final String WRONG2 = "wrong_2";
	private final String RIGHT = "right_1";
	private final String ID = "id";
	private final String SCORE = "score";
	private final String USERNAME = "username";
	/**
	 * Creates a QuestionDAO object to get the questions
	 * from the database.
	 * @param request
	 * @return
	 */
	public String getQuestions(HttpServletRequest request) {
		String data = "";
		List<Map> questions = null;

		try {
			QuestionDAO q = new QuestionDAO();
			questions = q.getQuestions(Integer.parseInt(request
					.getParameter("theme")), Integer.parseInt(request
					.getParameter("difficulty")));
		}
		catch (Exception e) {
			return "1";
		}
		Map m = null;
		data += questions.size() + "_";// Sets as the first characters the
		// amount of questions
		for(int i = 0; i < questions.size(); i++) {
			m = questions.get(i);
			data += (String) m.get(TEXT) + "|" + m.get(WRONG1) + "|"
					+ m.get(WRONG2) + "|" + m.get(RIGHT) + "|_";
		}
		return data;
	}
	/**
	 * Does the businesslogic and retrival of the highscores
	 * using highscoreDAO object to access the database.
	 * @param request
	 * @return
	 */
	public String getHighscore(HttpServletRequest request) {
		Integer userId = null;
		Integer userScore = null;
		String userName = null;
		if(request.getParameter("score") != null)
			userScore = Integer.parseInt(request.getParameter("score"));
		if(request.getParameter("user") != null)
			userName = request.getParameter("user");
		List<Map> highscores = null;
		String data = "";

		HighscoreDAO h = null;
		try {
			h = new HighscoreDAO();
		}
		catch (Exception e) {
			return "2";
		}
		if(userName != null && getHighscoreCount() < 10) {
			try {
				// TODO bugia puskee. highscoret ei ny
				userId = h.updateScore(0, userName, userScore);
			}
			catch (Exception e) {
				return "2";
			}
		}
		else {
			int[] minHighscore = getMinHighscore();
			if(minHighscore != null && userScore != null && userScore > minHighscore[1]) {
				try {
					userId = h.updateScore(minHighscore[0], userName, userScore);
				}
				catch (Exception e) {
					return "2";
				}
			}
			
		}
		if(h != null)
			highscores = h.getHighscores();

		Map m = null;
		for(int i = 0; i < highscores.size(); i++) {
			m = highscores.get(i);
			System.out.println("id = " + m.get(ID));
			data += (String) m.get(USERNAME) + "|" + m.get(SCORE) + "|_";
			if(userId != null && m.get(ID).equals(userId + ""))
				data = data.substring(0, data.length() - 1) + "*_";
			System.out.println("data = " + data);
		}
		return data;
	}
	/**
	 * Get's the minimum highscore from the database using
	 * HighScoreDAO to access the database.
	 * @return
	 */
	public int[] getMinHighscore() {
		List<Map> minHighscore = null;
		int[] data = new int[2];
		try {
			HighscoreDAO q = new HighscoreDAO();
			minHighscore = q.getMinHighscore();
		}
		catch (Exception e) {
			return null;
		}
		data[0] = Integer.parseInt(minHighscore.get(0).get(ID) + "");
		data[1] = Integer.parseInt(minHighscore.get(0).get(SCORE) + "");
		return data;
	}
	/**
	 * Get's the amount of highscores.
	 * @return
	 */
	public int getHighscoreCount() {
		int highscoreCount = 10;
		try {
			HighscoreDAO h = new HighscoreDAO();
			highscoreCount = Integer.parseInt(h.getHighscoreCount().get(0).get("count") + "");
		}
		catch (Exception e) {
			e.printStackTrace();
			return 10;
		}
		return highscoreCount;
	}
}
